@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkscrolledwindow.h>
+#include <gtk/gtklabel.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtksignal.h>
@@ -80,6 +81,7 @@ static GdkPixbuf *folderopen_pixbuf;
 static GdkPixbuf *foldernoselect_pixbuf;
 
 static GtkWidget *window;
+static GtkWidget *label;
 static GtkWidget *treeview;
 static GtkWidget *entry;
 static GtkWidget *confirm_area;
@@ -147,6 +149,15 @@ static gboolean tree_view_folder_item_func	(GtkTreeModel	  *model,
 
 FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 				 const gchar *default_folder)
+{
+	return foldersel_folder_sel_full(cur_folder, type, default_folder,
+					 NULL);
+}
+
+FolderItem *foldersel_folder_sel_full(Folder *cur_folder,
+				      FolderSelectionType type,
+				      const gchar *default_folder,
+				      const gchar *message)
 {
 	selected_item = NULL;
 	sel_type = type;
@@ -156,6 +167,12 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 		foldersel_init();
 	}
 
+	if (message) {
+		gtk_widget_show(label);
+		gtk_label_set_text(GTK_LABEL(label), message);
+	} else
+		gtk_widget_hide(label);
+
 	foldersel_set_tree(cur_folder);
 
 	/* select current */
@@ -176,6 +193,8 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 
 			selection = gtk_tree_view_get_selection
 				(GTK_TREE_VIEW(treeview));
+			gtkut_tree_view_expand_parent_all
+				(GTK_TREE_VIEW(treeview), &fis.iter);
 			gtk_tree_selection_select_iter(selection, &fis.iter);
 			gtk_tree_view_set_cursor(GTK_TREE_VIEW(treeview),
 						 fis.path, NULL, FALSE);
@@ -194,8 +213,8 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 	gtk_widget_grab_focus(ok_button);
 	gtk_widget_grab_focus(treeview);
 
-	gtk_widget_show(window);
 	manage_window_set_transient(GTK_WINDOW(window));
+	gtk_widget_show(window);
 
 	cancelled = finished = FALSE;
 
@@ -203,14 +222,17 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 		gtk_main_iteration();
 
 	gtk_widget_hide(window);
+	gtk_label_set_text(GTK_LABEL(label), "");
 	gtk_entry_set_text(GTK_ENTRY(entry), "");
 	gtk_tree_store_clear(tree_store);
 
 	if (cancelled || !selected_item)
 		return NULL;
 
-	if (type == FOLDER_SEL_MOVE_FOLDER ||
-	    (selected_item->path && !selected_item->no_select)) {
+	if (type == FOLDER_SEL_ALL ||
+	    (selected_item->stype != F_VIRTUAL &&
+	     (sel_type == FOLDER_SEL_MOVE_FOLDER ||
+	      (selected_item->path && !selected_item->no_select)))) {
 		folder_item = selected_item;
 		return folder_item;
 	}
@@ -229,7 +251,8 @@ static void foldersel_create(void)
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(window), _("Select folder"));
 	gtk_container_set_border_width(GTK_CONTAINER(window), 4);
-	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
+	gtk_window_set_position(GTK_WINDOW(window),
+				GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, FALSE);
 	gtk_window_set_wmclass
@@ -242,10 +265,15 @@ static void foldersel_create(void)
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
 
 	vbox = gtk_vbox_new(FALSE, 4);
+	gtk_widget_set_size_request(vbox, -1, 420);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
+	label = gtk_label_new("");
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 2);
+
 	scrolledwin = gtk_scrolled_window_new(NULL, NULL);
-	gtk_widget_set_size_request(scrolledwin, 300, 360);
+	gtk_widget_set_size_request(scrolledwin, 300, -1);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin),
 				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwin),
@@ -258,7 +286,7 @@ static void foldersel_create(void)
 					GDK_TYPE_PIXBUF,
 					GDK_TYPE_PIXBUF,
 					GDK_TYPE_COLOR,
-					G_TYPE_BOOLEAN);
+					G_TYPE_INT);
 	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(tree_store),
 					FOLDERSEL_FOLDERNAME,
 					foldersel_folder_name_compare,
@@ -284,6 +312,7 @@ static void foldersel_create(void)
 	column = gtk_tree_view_column_new();
 	gtk_tree_view_column_set_spacing(column, 2);
 	renderer = gtk_cell_renderer_pixbuf_new();
+	g_object_set(renderer, "ypad", 0, NULL);
 	gtk_tree_view_column_pack_start(column, renderer, FALSE);
 	gtk_tree_view_column_set_attributes
 		(column, renderer,
@@ -294,14 +323,14 @@ static void foldersel_create(void)
 
 	/* create text renderer */
 	renderer = gtk_cell_renderer_text_new();
+	g_object_set(renderer, "ypad", 0, NULL);
 	gtk_tree_view_column_pack_start(column, renderer, TRUE);
 	gtk_tree_view_column_set_attributes
 		(column, renderer,
 		 "text", FOLDERSEL_FOLDERNAME,
 		 "foreground-gdk", FOLDERSEL_FOREGROUND,
-		 "weight-set", FOLDERSEL_BOLD,
+		 "weight", FOLDERSEL_BOLD,
 		 NULL);
-	g_object_set(G_OBJECT(renderer), "weight", PANGO_WEIGHT_BOLD, NULL);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 
 	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
@@ -345,7 +374,9 @@ static void foldersel_append_item(GtkTreeStore *store, FolderItem *item,
 {
 	gchar *name;
 	GdkPixbuf *pixbuf, *pixbuf_open;
-	gboolean use_bold, use_color;
+	gboolean use_color;
+	gboolean no_select;
+	PangoWeight weight = PANGO_WEIGHT_NORMAL;
 	GdkColor *foreground = NULL;
 	static GdkColor color_noselect = {0, COLOR_DIM, COLOR_DIM, COLOR_DIM};
 	static GdkColor color_new = {0, (guint16)55000, 15000, 15000};
@@ -399,21 +430,26 @@ static void foldersel_append_item(GtkTreeStore *store, FolderItem *item,
 	} else
 		name = g_strdup(name);
 
-	pixbuf = item->no_select ? foldernoselect_pixbuf : folder_pixbuf;
-	pixbuf_open =
-		item->no_select ? foldernoselect_pixbuf : folderopen_pixbuf;
+	no_select = item->no_select ||
+		(sel_type != FOLDER_SEL_ALL && item->stype == F_VIRTUAL);
+
+	pixbuf = no_select ? foldernoselect_pixbuf : folder_pixbuf;
+	pixbuf_open = no_select ? foldernoselect_pixbuf : folderopen_pixbuf;
 
 	if (item->stype == F_OUTBOX || item->stype == F_DRAFT ||
 	    item->stype == F_TRASH) {
-		use_bold = use_color = FALSE;
+		use_color = FALSE;
 	} else if (item->stype == F_QUEUE) {
-		use_bold = use_color = (item->total > 0);
+		use_color = (item->total > 0);
+		if (item->total > 0)
+			weight = PANGO_WEIGHT_BOLD;
 	} else {
-		use_bold = (item->unread > 0);
 		use_color = (item->new > 0);
+		if (item->unread > 0)
+			weight = PANGO_WEIGHT_BOLD;
 	}
 
-	if (item->no_select)
+	if (no_select)
 		foreground = &color_noselect;
 	else if (use_color)
 		foreground = &color_new;
@@ -426,7 +462,7 @@ static void foldersel_append_item(GtkTreeStore *store, FolderItem *item,
 			   FOLDERSEL_PIXBUF, pixbuf,
 			   FOLDERSEL_PIXBUF_OPEN, pixbuf_open,
 			   FOLDERSEL_FOREGROUND, foreground,
-			   FOLDERSEL_BOLD, use_bold,
+			   FOLDERSEL_BOLD, weight,
 			   -1);
 	g_free(name);
 }
@@ -445,6 +481,15 @@ static void foldersel_insert_gnode_in_store(GtkTreeStore *store, GNode *node,
 	item = FOLDER_ITEM(node->data);
 	foldersel_append_item(store, item, &child, parent);
 
+	if (parent && item->parent && node->parent->children == node &&
+	    !item->parent->collapsed) {
+		GtkTreePath *path;
+
+		path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), parent);
+		gtk_tree_view_expand_row(GTK_TREE_VIEW(treeview), path, FALSE);
+		gtk_tree_path_free(path);
+	}
+
 	/* insert its children (this node as parent) */
 	for (iter = node->children; iter != NULL; iter = iter->next)
 		foldersel_insert_gnode_in_store(store, iter, &child);
@@ -472,8 +517,6 @@ static void foldersel_set_tree(Folder *cur_folder)
 	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(tree_store),
 					     FOLDERSEL_FOLDERNAME,
 					     GTK_SORT_ASCENDING);
-
-	gtk_tree_view_expand_all(GTK_TREE_VIEW(treeview));
 }
 
 static gboolean foldersel_selected(GtkTreeSelection *selection,
@@ -494,8 +537,10 @@ static gboolean foldersel_selected(GtkTreeSelection *selection,
 
 	selected_item = item;
 	if (selected_item &&
-	    (sel_type == FOLDER_SEL_MOVE_FOLDER ||
-	     (selected_item->path && !selected_item->no_select))) {
+	    (sel_type == FOLDER_SEL_ALL ||
+	     (selected_item->stype != F_VIRTUAL &&
+	      (sel_type == FOLDER_SEL_MOVE_FOLDER ||
+	       (selected_item->path && !selected_item->no_select))))) {
 		gchar *id;
 		id = folder_item_get_identifier(selected_item);
 		gtk_entry_set_text(GTK_ENTRY(entry), id);
