@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -93,12 +93,17 @@
 #  include <windows.h>
 #  include <pbt.h>
 #  include <fcntl.h>
+#  include <conio.h>
 #endif
 
 #include "version.h"
 
 gchar *prog_version;
 
+#ifdef G_OS_WIN32
+static gboolean init_console_done = FALSE;
+#endif
+
 static gint lock_socket = -1;
 static gint lock_socket_tag = 0;
 static GIOChannel *lock_ch = NULL;
@@ -130,9 +135,7 @@ static void parse_gtkrc_files		(void);
 static void setup_rc_dir		(void);
 static void check_gpg			(void);
 static void set_log_handlers		(gboolean	 enable);
-#ifdef G_OS_WIN32
-static void process_win32_message	(void);
-#endif
+static void register_system_events	(void);
 
 static gchar *get_socket_name		(void);
 static gint prohibit_duplicate_launch	(void);
@@ -298,10 +301,6 @@ int main(int argc, char *argv[])
 
 	set_log_handlers(TRUE);
 
-#ifdef G_OS_WIN32
-	process_win32_message();
-#endif
-
 	account_read_config_all();
 	account_set_menu();
 	main_window_reflect_prefs_all();
@@ -321,6 +320,8 @@ int main(int argc, char *argv[])
 
 	addressbook_read_file();
 
+	register_system_events();
+
 	inc_autocheck_timer_init(mainwin);
 
 	remote_command_exec();
@@ -335,7 +336,6 @@ static void init_console(void)
 #ifdef G_OS_WIN32
 	gint fd;
 	FILE *fp;
-	static gboolean init_console_done = FALSE;
 
 	if (init_console_done)
 		return;
@@ -591,7 +591,9 @@ static void app_init(void)
 	g_unsetenv("LANGUAGE");
 #endif
 
+#ifdef HAVE_LOCALE_H
 	setlocale(LC_ALL, "");
+#endif
 
 	prog_version = PROG_VERSION;
 	set_startup_dir();
@@ -603,6 +605,7 @@ static void app_init(void)
 	g_free(newpath);
 #endif
 
+#ifdef ENABLE_NLS
 	if (g_path_is_absolute(LOCALEDIR))
 		bindtextdomain(PACKAGE, LOCALEDIR);
 	else {
@@ -621,18 +624,14 @@ static void app_init(void)
 				locale_dir = locale_dir_;
 			}
 		}
-#endif
+#endif /* G_OS_WIN32 */
 		bindtextdomain(PACKAGE, locale_dir);
 		g_free(locale_dir);
 	}
 
 	bind_textdomain_codeset(PACKAGE, CS_UTF_8);
 	textdomain(PACKAGE);
-
-#ifdef G_OS_UNIX
-	/* ignore SIGPIPE signal for preventing sudden death of program */
-	signal(SIGPIPE, SIG_IGN);
-#endif
+#endif /* ENABLE_NLS */
 
 #ifdef G_OS_WIN32
 	read_ini_file();
@@ -829,9 +828,11 @@ static void check_gpg(void)
 		/* Also does some gpgme init */
 	        gpgme_engine_info_t engineInfo;
 
+#if HAVE_LOCALE_H
 		gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
 		gpgme_set_locale(NULL, LC_MESSAGES,
 				 setlocale(LC_MESSAGES, NULL));
+#endif
 
 		if (!gpgme_get_engine_info(&engineInfo)) {
 			while (engineInfo) {
@@ -967,6 +968,15 @@ static void set_log_handlers(gboolean enable)
 }
 
 #ifdef G_OS_WIN32
+static BOOL WINAPI
+ctrl_handler(DWORD dwctrltype)
+{
+	log_print("ctrl_handler: received %d\n", dwctrltype);
+	app_will_exit(TRUE);
+
+	return TRUE;
+}
+
 static LRESULT CALLBACK
 wndproc(HWND hwnd, UINT message, WPARAM wparam, LPARAM lparam)
 {
@@ -996,16 +1006,26 @@ wndproc(HWND hwnd, UINT message, WPARAM wparam, LPARAM lparam)
 	return DefWindowProc(hwnd, message, wparam, lparam);
 }
 
-static void process_win32_message(void)
+static void register_system_events(void)
 {
 	WNDCLASS wclass;
 	static HWND hwnd = NULL;
+	static BOOL ctrl_handler_set = FALSE;
 	ATOM klass;
 	HINSTANCE hmodule = GetModuleHandle(NULL);
 
+	if (init_console_done && !ctrl_handler_set) {
+		debug_print("register_system_events(): SetConsoleCtrlHandler\n");
+		ctrl_handler_set = SetConsoleCtrlHandler(ctrl_handler, TRUE);
+		if (!ctrl_handler_set)
+			g_warning("SetConsoleCtrlHandler() failed\n");
+	}
+
 	if (hwnd)
 		return;
 
+	debug_print("register_system_events(): RegisterClass\n");
+
 	memset(&wclass, 0, sizeof(WNDCLASS));
 	wclass.lpszClassName = "sylpheed-observer";
 	wclass.lpfnWndProc   = wndproc;
@@ -1020,6 +1040,44 @@ static void process_win32_message(void)
 	if (!hwnd)
 		UnregisterClass(MAKEINTRESOURCE(klass), hmodule);
 }
+#else /* G_OS_WIN32 */
+static void sig_handler(gint signum)
+{
+	debug_print("signal %d received\n", signum);
+
+	switch (signum) {
+	case SIGHUP:
+	case SIGINT:
+	case SIGTERM:
+	case SIGQUIT:
+		app_will_exit(TRUE);
+		break;
+	default:
+		break;
+	}
+}
+
+static void register_system_events(void)
+{
+	struct sigaction sa;
+
+	memset(&sa, 0, sizeof(sa));
+	sa.sa_handler = sig_handler;
+	sa.sa_flags = SA_RESTART;
+
+	sigemptyset(&sa.sa_mask);
+	sigaddset(&sa.sa_mask, SIGHUP);
+	sigaddset(&sa.sa_mask, SIGINT);
+	sigaddset(&sa.sa_mask, SIGTERM);
+	sigaddset(&sa.sa_mask, SIGQUIT);
+	sigaddset(&sa.sa_mask, SIGPIPE);
+
+	sigaction(SIGHUP, &sa, NULL);
+	sigaction(SIGINT, &sa, NULL);
+	sigaction(SIGTERM, &sa, NULL);
+	sigaction(SIGQUIT, &sa, NULL);
+	sigaction(SIGPIPE, &sa, NULL);
+}
 #endif
 
 static gchar *get_socket_name(void)
