@@ -602,10 +602,10 @@ gint syldap_search( SyldapServer *ldapServer ) {
 			if( g_ascii_strcasecmp( attribute, SYLDAP_ATTR_DN ) == 0 ) {
 				listDN = syldap_add_single_value( ld, e, attribute );
 			}
-		}
 
-		/* Free memory used to store attribute */
-		ldap_memfree( attribute );
+			/* Free memory used to store attribute */
+			ldap_memfree( attribute );
+		}
 
 		/* Format and add items to cache */
 		syldap_build_items_fl( ldapServer, listAddress, listFirst, listLast );
@@ -637,16 +637,15 @@ gint syldap_search( SyldapServer *ldapServer ) {
 
 /* syldap_display_search_results() - updates the ui. this function is called from the
  * main thread (the thread running the GTK event loop). */
-static gint syldap_display_search_results(SyldapServer *ldapServer)
+static gboolean syldap_display_search_results(SyldapServer *ldapServer)
 {
 	/* NOTE: when this function is called the accompanying thread should
 	 * already be terminated. */
-	gtk_idle_remove(ldapServer->idleId);
 	ldapServer->callBack(ldapServer);
 	/* FIXME:  match should know whether to free this SyldapServer stuff. */
 	g_free(ldapServer->thread);
 	ldapServer->thread = NULL;
-	return TRUE;
+	return FALSE;
 }
 
 /* ============================================================================================ */
@@ -821,8 +820,8 @@ GList *syldap_read_basedn_s( const gchar *host, const gint port, const gchar *bi
 					}
 					ldap_value_free( vals );
 				}
+				ldap_memfree( attribute );
 			}
-			ldap_memfree( attribute );
 			if( ber != NULL ) {
 				ber_free( ber, 0 );
 			}
@@ -863,8 +862,8 @@ GList *syldap_read_basedn_s( const gchar *host, const gint port, const gchar *bi
 						}
 						ldap_value_free( vals );
 					}
+					ldap_memfree( attribute );
 				}
-				ldap_memfree( attribute );
 				if( ber != NULL ) {
 					ber_free( ber, 0 );
 				}
@@ -950,8 +949,8 @@ GList *syldap_read_basedn( SyldapServer *ldapServer ) {
 					}
 					ldap_value_free( vals );
 				}
+				ldap_memfree( attribute );
 			}
-			ldap_memfree( attribute );
 			if( ber != NULL ) {
 				ber_free( ber, 0 );
 			}
@@ -994,8 +993,8 @@ GList *syldap_read_basedn( SyldapServer *ldapServer ) {
 						}
 						ldap_value_free( vals );
 					}
+					ldap_memfree( attribute );
 				}
-				ldap_memfree( attribute );
 				if( ber != NULL ) {
 					ber_free( ber, 0 );
 				}
